@@ -3,7 +3,14 @@
 #' @description Fit the one-step Šesták–Berggren kinetic model.
 #'
 #' @details Fit the one-step Šesták–Berggren kinetic (non-linear) model using
-#'  accelerated stability data from an R dataframe format. Parameters are kept in even when not significant.
+#' accelerated stability data that has been stored in an R data frame. Additionally,
+#' predictions of the mean at each tested temperature are returned, including associated
+#' confidence and prediction intervals, which can be subsequently visualised with
+#' step1_plot_pred(), step1_plot_CI(), step1_plot_PI() and step1_plot_T(). Kinetic
+#' parameters (k1, k2 and, if used, k3) are retained in the model even if one or more of
+#' these parameters turn out to be non-significant. Further arguments relating to
+#' model fitting, such as setting lower bounds for one or more model parameters,
+#' may be passed.
 #'
 #' @param data Dataframe containing accelerated stability data (required).
 #' @param y Name of decreasing variable (e.g. concentration) contained within data
@@ -11,8 +18,10 @@
 #' @param .time Time variable contained within data (required).
 #' @param K Kelvin variable (numeric or column name) (optional).
 #' @param C Celsius variable (numeric or column name) (optional).
-#' @param validation Validation dummy variable, the column must contain only 1s and 0s, 1 for validation data and 0 for fit data. (column name) (optional).
-#' @param draw Number of simulations used to estimate confidence intervals. When set to NULL the calculus method is used, however this is not recommended.
+#' @param validation Validation dummy variable, the column must contain only
+#'  1s and 0s, 1 for validation data and 0 for fit data. (column name) (optional).
+#' @param draw Number of simulations used to estimate confidence intervals.
+#'  When set to NULL the calculus method is used, however this is not recommended.
 #' @param parms Starting values for the parameters as a list - k1, k2, k3, and c0.
 #' @param temp_pred_C Integer or numeric value to predict the response for a
 #'  given temperature (in Celsius).
@@ -24,7 +33,7 @@
 #' @param reparameterisation Use alternative parameterisation of the one-step
 #'  model which aims to reduce correlation between k1 and k2.
 #' @param zero_order Set kinetic order, k3, to zero (straight lines).
-
+#' @param ... Further arguments to passed to minpack.lm.
 #'
 #' @return An SB class object, a list including the following elements:
 #' \itemize{
@@ -55,6 +64,10 @@
 #' fit4 <- step1_down(data = antigenicity, y = "conc", .time = "time",C = "Celsius",
 #'   reparameterisation = TRUE, draw = 5000)
 #'
+#' #Use a custom lower bound for k1 (default is 0).
+#' fit5 <- step1_down(data = potency, y = "Potency", .time = "Time",C = "Celsius",
+#'   reparameterisation = TRUE, zero_order = TRUE, draw = 5000, lower = c(-Inf, 0, 0))
+#'
 #' @importFrom stats vcov coef runif confint rnorm quantile qt complete.cases
 #' @importFrom minpack.lm nls.lm
 #' @importFrom mvtnorm rmvt
@@ -64,45 +77,48 @@
 step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
                         draw = 10000, parms = NULL, temp_pred_C = NULL,
                         max_time_pred = NULL, confidence_interval = 0.95, by = 101,
-                        reparameterisation = FALSE, zero_order = FALSE){
+                        reparameterisation = FALSE, zero_order = FALSE, ...){
 
   if (is.null(K) & is.null(C))
     stop("Select the temperature variable in Kelvin or Celsius")
+
   if (!is.null(parms) & !is.list(parms))
     stop("The starting values for parameters must be a list, or keep as NULL")
 
+  if (!is.null(validation))
+    if (!all(data[,validation] %in% c(0,1)))
+      stop("Validation column must contain 1s and 0s only")
+
   user_parameters <- list(
     data = data, y = y, .time = .time, K = K, C = C, validation = validation,draw = draw,
     parms = parms, temp_pred_C = temp_pred_C, max_time_pred = max_time_pred,
     confidence_interval = confidence_interval, by = by,
-    reparameterisation = reparameterisation, zero_order = zero_order
-  )
+    reparameterisation = reparameterisation, zero_order = zero_order)
 
+  ## Additional arguments in the call will be passed to model fitting with minipak.lm
+  minpack_args = list(...)                    ##
+
+  ## Temperature: both C and K are provided
   if(!is.null(C) & !is.null(K)) {
-
     data[, C] <- ifelse(is.na(data[, C]) & !is.na(data[, K]),
-                        data$K - 273.15,
+                        data[, K] - 273.15,
                         data[, C])
-
     data[, K] <- ifelse(is.na(data[, K]) & !is.na(data[, C]),
-                        data$C + 273.15,
-                        data[, K])
-  }
+                        data[, C] + 273.15,
+                        data[, K])   }
+
+  ## Temperature: only C or only K is provided
+  if (!is.null(C) & is.null(K)) {             ##
+   K = 'K'                                  ##
+   data[, K] = data[, C] + 273.15  }        ##
+  if (!is.null(K) & is.null(C)) {             ##
+    C = 'C'                                 ##
+    data[, C] = data[, K] - 273.15 }        ##
 
   data <- data[complete.cases(data[, c(C,K,y,.time)]), ]
 
   dat = data
-
-  if (!is.null(validation))
-    if (!all(dat[,validation] %in% c(0,1)))
-      stop("Validation column must contain 1s and 0s only")
-
-  if (is.null(K))
-    dat$K = dat[, C] + 273.15
-  if (is.null(C)) {
-    dat$C = dat[, K] - 273.15
-    C = "C"}
-
+  dat$K = dat[, K]                         ##
   Kref = mean(dat$K)
   dat$Celsius = as.factor(dat[, C])
   dat$time = dat[, .time]
@@ -145,8 +161,13 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
     c0_initial <- mean(sorted_data$y[selected_rows])
   }
 
-  if(reparameterisation & zero_order){ # reparameterisation and k3 is 0
-    MyFctNL = function(parms) { # Make function
+## Model type 1 - reparameterisation and k3 = 0
+  if(reparameterisation & zero_order){
+
+## Print a message informing lower bounds = 0 may not be suitable with the reparamerised version
+cat("The alternative parameterisation of the one-step model was used. Note that the lower bounds for all parameters are set to 0 unless other lower bounds are specified in step1_down() or step1_down_basic().\n\n")
+
+  MyFctNL = function(parms) { # Make function
       k1 = parms$k1
       k2 = parms$k2
       c0 = parms$c0
@@ -156,21 +177,38 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
       return(residual)
     }
 
+  if (!"fn" %in% names(minpack_args)) {	##
+    minpack_args$fn =  MyFctNL    }     ##
+
     # Fit model :
     if (!is.null(parms)) {
-      fit = minpack.lm::nls.lm(par = parms, fn = MyFctNL, lower = rep(0,
-                                                                      length(parms)))
+     minpack_args$par =  parms                          ##
+    if (!"lower" %in% names(minpack_args)) 	{	##
+    minpack_args$lower =  rep(0, length(parms))   }     ##
+
+	if(length(minpack_args$par) != length(minpack_args$lower))                             ##
+	stop("The number of parameters (",length(minpack_args$par),") does not match the number of specified lower bounds (",length(minpack_args$lower),").")  ##
+
+  fit = do.call(minpack.lm::nls.lm, minpack_args)
     }
     else {
       repeat {
         suppressWarnings(rm(fit))
 
-        parms = list(k1 = stats::runif(1, 0, 40), k2 = stats::runif(1,
-                                                                    1000, 20000), c0 = c0_initial)
-        fit = suppressWarnings(minpack.lm::nls.lm(par = parms,
-                                                  fn = MyFctNL, lower = rep(0, length(parms))))
-        fit <- tryCatch({
-          suppressWarnings(minpack.lm::nls.lm(par = parms, fn = MyFctNL, lower = rep(0, length(parms))))
+        parms = list(k1 = stats::runif(1, 0, 40), k2 = stats::runif(1, 1000, 20000), c0 = c0_initial)
+
+minpack_args$par = parms
+
+if (!"lower" %in% names(minpack_args)) 	{	##
+	minpack_args$lower =  rep(0, length( parms ))   }     ##
+
+	if(length(minpack_args$par) != length(minpack_args$lower))                             ##
+	stop("The number of parameters (",length(minpack_args$par),") does not match the number of specified lower bounds (",length(minpack_args$lower),").")  ##
+
+  fit = suppressWarnings(do.call(minpack.lm::nls.lm, minpack_args))
+
+   fit <- tryCatch({
+          suppressWarnings(do.call(minpack.lm::nls.lm, minpack_args))
         },
         error = function(e){"error"},
         warning = function(w){"warning"})
@@ -185,9 +223,9 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
           break
         }
       }
-      fit = minpack.lm::nls.lm(par = parms, fn = MyFctNL, lower = rep(0,
-                                                                      length(parms)))
-    }
+      fit = do.call(minpack.lm::nls.lm, minpack_args)
+
+ }
 
     # Calculate the predictions
     k1 = stats::coef(fit)[1]
@@ -217,7 +255,7 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
         return(conc)
       }
       # Multi T bootstrap
-      rand.coef = rmvt(draw, sigma = SIG, df = nrow(dat) - 3) + matrix(nrow = draw, ncol = 3, byrow = TRUE, coef(fit))
+      rand.coef = mvtnorm::rmvt(draw, sigma = SIG, df = nrow(dat) - 3) + matrix(nrow = draw, ncol = 3, byrow = TRUE, coef(fit))
       res.boot = matrix(nrow = draw, ncol = nrow(pred), byrow = TRUE, apply(rand.coef, 1, pred_fct))
 
       CI1b = apply(res.boot, 2, quantile, ((1-confidence_interval)/2), na.rm = TRUE)
@@ -226,11 +264,10 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
       res.boot = res.boot + rnorm(draw*length(pred$time), 0, sigma)
       PI1b = apply(res.boot, 2, quantile, ((1-confidence_interval)/2), na.rm = TRUE)
       PI2b = apply(res.boot, 2, quantile, ((1+confidence_interval)/2), na.rm = TRUE)
-
-
     }
 
-  }else if(!reparameterisation & zero_order){ # no reparameterisation and k3 is 0
+## Model type 2 - no reparameterisation and k3 = 0
+  }else if(!reparameterisation & zero_order){
     MyFctNL = function(parms) { # make function
       k1 = parms$k1
       k2 = parms$k2
@@ -242,18 +279,36 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
       residual = dat$y - Model
       return(residual)
     }
+
+  if (!"fn" %in% names(minpack_args)) 	{	##
+    minpack_args$fn =  MyFctNL    }             ##
+
     if (!is.null(parms)) { # fit model
-      fit = minpack.lm::nls.lm(par = parms, fn = MyFctNL, lower = rep(0,
-                                                                      length(parms)))
+    minpack_args$par =  parms                       ##
+    if (!"lower" %in% names(minpack_args)) 	{   ##
+    minpack_args$lower =  rep(0, length(parms))   } ##
+
+	if(length(minpack_args$par) != length(minpack_args$lower))                             ##
+	stop("The number of parameters (",length(minpack_args$par),") does not match the number of specified lower bounds (",length(minpack_args$lower),").")  ##
+
+      fit = do.call(minpack.lm::nls.lm, minpack_args)
     }
     else {
       repeat {
         suppressWarnings(rm(fit))
 
-        parms = list(k1 = stats::runif(1, 0, 40), k2 = stats::runif(1,
-                                                                    1000, 20000), c0 = c0_initial)
-        fit <- tryCatch({
-          suppressWarnings(minpack.lm::nls.lm(par = parms, fn = MyFctNL, lower = rep(0, length(parms))))
+        parms = list(k1 = stats::runif(1, 0, 40), k2 = stats::runif(1, 1000, 20000), c0 = c0_initial)
+
+  minpack_args$par = parms ##
+
+if (!"lower" %in% names(minpack_args)) 	{	##
+	    minpack_args$lower =  rep(0, length( parms ))   }   ##
+
+	if(length(minpack_args$par) != length(minpack_args$lower))                             ##
+  stop("The number of parameters (",length(minpack_args$par),") does not match the number of specified lower bounds (",length(minpack_args$lower),").")  ##
+
+ fit <- tryCatch({
+          suppressWarnings(do.call(minpack.lm::nls.lm, minpack_args))
         },
         error = function(e){"error"},
         warning = function(w){"warning"})
@@ -268,10 +323,10 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
           break
         }
       }
-      fit = minpack.lm::nls.lm(par = parms, fn = MyFctNL, lower = rep(0,
-                                                                      length(parms)))
+      fit = do.call(minpack.lm::nls.lm, minpack_args)
     }
-    # Predict
+
+# Predict
     k1 = coef(fit)[1]
     k2 = coef(fit)[2]
     c0 = coef(fit)[3]
@@ -299,7 +354,7 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
         return(conc)
       }
       # Multi T bootstrap
-      rand.coef = rmvt(draw, sigma = SIG, df = nrow(dat) - 3) + matrix(nrow = draw, ncol = 3, byrow = TRUE, coef(fit))
+      rand.coef = mvtnorm::rmvt(draw, sigma = SIG, df = nrow(dat) - 3) + matrix(nrow = draw, ncol = 3, byrow = TRUE, coef(fit))
       res.boot = matrix(nrow = draw, ncol = nrow(pred), byrow = TRUE, apply(rand.coef, 1, pred_fct))
 
       CI1b = apply(res.boot, 2, quantile, ((1-confidence_interval)/2), na.rm = TRUE)
@@ -312,9 +367,13 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
 
     }
 
+## Model type 3 - reparameterisation and k3 is not zero
+  }else if(reparameterisation & !zero_order){
 
-  }else if(reparameterisation & !zero_order){ #reparameterisation and k3 is not zero
-    MyFctNL = function(parms) {
+  ## Print a message informing lower bounds = 0 may not be suitable with the reparamerised version
+  cat("The alternative parameterisation of the one-step model was used. Note that the lower bounds for all parameters are set to 0 unless other lower bounds are specified in step1_down() or step1_down_basic().\n\n")
+
+   MyFctNL = function(parms) {
       k1 = parms$k1
       k2 = parms$k2
       k3 = parms$k3
@@ -324,18 +383,36 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
       residual = dat$y - Model
       return(residual)
     }
-    if (!is.null(parms)) { # Fit the model
-      fit = minpack.lm::nls.lm(par = parms, fn = MyFctNL, lower = rep(0,
-                                                                      length(parms)))
+
+  if (!"fn" %in% names(minpack_args)) 	{	##
+    minpack_args$fn =  MyFctNL    }             ##
+
+  if (!is.null(parms)) { # Fit the model
+        minpack_args$par =  parms               ##
+  if (!"lower" %in% names(minpack_args)) 	{	##
+    minpack_args$lower =  rep(0, length(parms))   }     ##
+
+	if(length(minpack_args$par) != length(minpack_args$lower))                             ##
+  stop("The number of parameters (",length(minpack_args$par),") does not match the number of specified lower bounds (",length(minpack_args$lower),").")  ##
+
+      fit = do.call(minpack.lm::nls.lm, minpack_args)
     }
     else {
       repeat {
         suppressWarnings(rm(fit))
 
-        parms = list(k1 = stats::runif(1, 0, 60), k2 = stats::runif(1,
-                                                                    1000, 20000), k3 = stats::runif(1, 0, 11), c0 = c0_initial)
-        fit <- tryCatch({
-          suppressWarnings(minpack.lm::nls.lm(par = parms, fn = MyFctNL, lower = rep(0, length(parms))))
+        parms = list(k1 = stats::runif(1, 0, 60), k2 = stats::runif(1, 1000, 20000), k3 = stats::runif(1, 0, 11), c0 = c0_initial)
+
+  minpack_args$par = parms
+
+  if (!"lower" %in% names(minpack_args)) 	{	##
+	    minpack_args$lower =  rep(0, length( parms ))   }     ##
+
+	if(length(minpack_args$par) != length(minpack_args$lower))                             ##
+  stop("The number of parameters (",length(minpack_args$par),") does not match the number of specified lower bounds (",length(minpack_args$lower),").")  ##
+
+   fit <- tryCatch({
+          suppressWarnings(do.call(minpack.lm::nls.lm, minpack_args))
         },
         error = function(e){"error"},
         warning = function(w){"warning"})
@@ -350,9 +427,8 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
           break
         }
       }
-      fit = minpack.lm::nls.lm(par = parms, fn = MyFctNL, lower = rep(0,
-                                                                      length(parms)))
-    }
+      fit = do.call(minpack.lm::nls.lm, minpack_args)
+ }
 
     # Predict
     k1 = coef(fit)[1]
@@ -386,7 +462,7 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
         return(conc)
       }
       # Multi T bootstrap
-      rand.coef = rmvt(draw, sigma = SIG, df = nrow(dat) - 4) + matrix(nrow = draw, ncol = 4, byrow = TRUE, coef(fit))
+      rand.coef = mvtnorm::rmvt(draw, sigma = SIG, df = nrow(dat) - 4) + matrix(nrow = draw, ncol = 4, byrow = TRUE, coef(fit))
       res.boot = matrix(nrow = draw, ncol = nrow(pred), byrow = TRUE, apply(rand.coef, 1, pred_fct))
 
       no_k3_below0 <- sum(rand.coef[,3] < 0)
@@ -404,7 +480,8 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
 
     }
 
-  }else if(!reparameterisation & !zero_order){ # No re-parameterisation and k3 not zero
+## Model type 4 - no reparameterisation and k3 is not 0
+  }else if(!reparameterisation & !zero_order){
     MyFctNL = function(parms) {
       k1 = parms$k1
       k2 = parms$k2
@@ -417,18 +494,36 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
       return(residual)
 
     }
+
+  if (!"fn" %in% names(minpack_args)) 	{	##
+    minpack_args$fn =  MyFctNL    }             ##
+
     if (!is.null(parms)) { # Fitting the model
-      fit = minpack.lm::nls.lm(par = parms, fn = MyFctNL, lower = rep(0,
-                                                                      length(parms)))
+    minpack_args$par =  parms                   ##
+  if (!"lower" %in% names(minpack_args)) 	{	##
+    minpack_args$lower =  rep(0, length(parms))   }   ##
+
+	if(length(minpack_args$par) != length(minpack_args$lower))                             ##
+  stop("The number of parameters (",length(minpack_args$par),") does not match the number of specified lower bounds (",length(minpack_args$lower),").")  ##
+
+      fit = do.call(minpack.lm::nls.lm, minpack_args)
     }
     else {
       repeat {
         suppressWarnings(rm(fit))
 
-        parms = list(k1 = stats::runif(1, 0, 60), k2 = stats::runif(1,
-                                                                    1000, 20000), k3 = stats::runif(1, 0, 11), c0 = c0_initial)
-        fit <- tryCatch({
-          suppressWarnings(minpack.lm::nls.lm(par = parms, fn = MyFctNL, lower = rep(0, length(parms))))
+        parms = list(k1 = stats::runif(1, 0, 60), k2 = stats::runif(1, 1000, 20000), k3 = stats::runif(1, 0, 11), c0 = c0_initial)
+
+  minpack_args$par = parms
+
+  if (!"lower" %in% names(minpack_args)) 	{	##
+	    minpack_args$lower =  rep(0, length( parms ))   } ##
+
+	if(length(minpack_args$par) != length(minpack_args$lower))                             ##
+  stop("The number of parameters (",length(minpack_args$par),") does not match the number of specified lower bounds (",length(minpack_args$lower),").")  ##
+
+ fit <- tryCatch({
+          suppressWarnings(do.call(minpack.lm::nls.lm, minpack_args))
         },
         error = function(e){"error"},
         warning = function(w){"warning"})
@@ -443,9 +538,9 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
           break
         }
       }
-      fit = minpack.lm::nls.lm(par = parms, fn = MyFctNL, lower = rep(0,
-                                                                      length(parms)))
+      fit = do.call(minpack.lm::nls.lm, minpack_args)
     }
+
     # Predict
     k1 = coef(fit)[1]
     k2 = coef(fit)[2]
@@ -477,7 +572,7 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
         return(conc)
       }
       # Multi T bootstrap
-      rand.coef = rmvt(draw, sigma = SIG, df = nrow(dat) - 4) + matrix(nrow = draw, ncol = 4, byrow = TRUE, coef(fit))
+      rand.coef = mvtnorm::rmvt(draw, sigma = SIG, df = nrow(dat) - 4) + matrix(nrow = draw, ncol = 4, byrow = TRUE, coef(fit))
       res.boot = matrix(nrow = draw, ncol = nrow(pred), byrow = TRUE, apply(rand.coef, 1, pred_fct))
 
       no_k3_below0 <- sum(rand.coef[,3] < 0)
@@ -529,7 +624,3 @@ step1_down <- function (data, y, .time, K = NULL, C = NULL, validation = NULL,
   return(results)
 
   }
-
-
-
-
