@@ -17,19 +17,21 @@
 #' @return ggplot2 object with focus on chosen temperature.
 #'
 #' @examples
-#' #load potency data
+#' #Load potency data
 #' data(potency)
 #'
-#' #run step1_down fit
+#' #Run step1_down fit
 #' fit1 <- step1_down(data = potency, y = "Potency", .time = "Time",
 #'  C = "Celsius", zero_order = TRUE)
 #'
-#' #plot raw data with prediction curves with focus on temperature in dataset.
-#' step1_plot_T(fit1, focus_T = 5,ribbon = TRUE, xlim = NULL, ylim = c(0,12),
+#' #Plot raw data with prediction curves with focus on temperature in dataset.
+#' #Also limit x-axis to values between 0 and 10 and limit y-axis to values between 0 and 12.
+#' step1_plot_T(fit1, focus_T = 5,ribbon = TRUE, xlim = c(0,10), ylim = c(0,12),
 #'  xname = "Time (Month)", yname = "Potency")
 #'
-#' #plot raw data with prediction curves with focus on temperature not in dataset.
-#' step1_plot_T(fit1, focus_T = -10,ribbon = TRUE, xlim = NULL, ylim = c(0,12),
+#' #Plot raw data with prediction curves with focus on temperature not in dataset.
+#' #Also limit x-axis to values between 0 and 10 and limit y-axis to values between 0 and 12.
+#' step1_plot_T(fit1, focus_T = -10,ribbon = TRUE, xlim = c(0,10), ylim = c(0,12),
 #'  xname = "Time (Months)", yname = "Potency")
 #'
 #' @import ggplot2
@@ -101,8 +103,9 @@ step1_plot_T <- function (step1_down_object, focus_T = NULL, xname = NULL, yname
 
   plot = ggplot() +
    labs( x = xname, y = yname) +
-    {if(!is.null(xlim))scale_x_continuous(limits = xlim)} +
-    {if(!is.null(ylim))scale_y_continuous(limits = ylim)} +
+   {if(!is.null(ylim)& is.null(xlim))coord_cartesian(ylim = ylim)} +
+   {if(is.null(ylim)& !is.null(xlim))coord_cartesian(xlim = xlim)} +
+   {if(!is.null(xlim) & !is.null(ylim))coord_cartesian(xlim = xlim, ylim = ylim)} +
    mytheme  +
    geom_line(data=pred, mapping=aes(x= time, y = Response, colour = Celsius, linetype = "Prediction")) +
    geom_line(data=pred[xx,], mapping=aes(x= time, y = CI1, colour = Celsius, linetype = confidence_i)) +
